@@ -1,10 +1,10 @@
-/* desktop recorder
- * Copyright (C) 2005 Benjamin Otte <otte@gnome.org
+/* desktop session
+ * Copyright (C) 2005 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,13 +21,14 @@
 #  include "config.h"
 #endif
 
-#include "byzanzrecorder.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+
+#include "byzanzsession.h"
 
 static int duration = 10;
 static int delay = 1;
-static gboolean loop = FALSE;
 static gboolean cursor = FALSE;
+static gboolean audio = FALSE;
 static gboolean verbose = FALSE;
 static GdkRectangle area = { 0, 0, G_MAXINT / 2, G_MAXINT / 2 };
 
@@ -35,8 +36,8 @@ static GOptionEntry entries[] =
 {
   { "duration", 'd', 0, G_OPTION_ARG_INT, &duration, N_("Duration of animation (default: 10 seconds)"), N_("SECS") },
   { "delay", 0, 0, G_OPTION_ARG_INT, &delay, N_("Delay before start (default: 1 second)"), N_("SECS") },
-  { "loop", 'l', 0, G_OPTION_ARG_NONE, &loop, N_("Let the animation loop"), NULL },
   { "cursor", 'c', 0, G_OPTION_ARG_NONE, &cursor, N_("Record mouse cursor"), NULL },
+  { "audio", 'a', 0, G_OPTION_ARG_NONE, &audio, N_("Record audio"), NULL },
   { "x", 'x', 0, G_OPTION_ARG_INT, &area.x, N_("X coordinate of rectangle to record"), N_("PIXEL") },
   { "y", 'y', 0, G_OPTION_ARG_INT, &area.y, N_("Y coordinate of rectangle to record"), N_("PIXEL") },
   { "width", 'w', 0, G_OPTION_ARG_INT, &area.width, N_("Width of recording rectangle"), N_("PIXEL") },
@@ -67,53 +68,72 @@ usage (void)
   g_print (_("       %s --help\n"), g_get_prgname ());
 }
 
-static gboolean
-stop_recording (gpointer recorder)
+static void
+session_notify_cb (ByzanzSession *session, GParamSpec *pspec, gpointer unused)
 {
-  verbose_print (_("Recording done. Cleaning up...\n"));
-  byzanz_recorder_stop (recorder);
-  gtk_main_quit ();
+  const GError *error = byzanz_session_get_error (session);
+  
+  if (g_str_equal (pspec->name, "error")) {
+    g_print (_("Error during recording: %s\n"), error->message);
+    gtk_main_quit ();
+    return;
+  }
+
+  if (!byzanz_session_is_encoding (session)) {
+    verbose_print (_("Recording done.\n"));
+    gtk_main_quit ();
+  }
+}
+
+static gboolean
+stop_recording (gpointer session)
+{
+  verbose_print (_("Recording completed. Finishing encoding...\n"));
+  byzanz_session_stop (session);
   
   return FALSE;
 }
 
 static gboolean
-start_recording (gpointer recorder)
+start_recording (gpointer session)
 {
   verbose_print (_("Recording starts. Will record %d seconds...\n"), duration / 1000);
-  byzanz_recorder_start (recorder);
-  g_timeout_add (duration, stop_recording, recorder);
+  byzanz_session_start (session);
+  g_timeout_add (duration, stop_recording, session);
   
   return FALSE;
 }
 
 static gboolean
-prepare_recording (gpointer recorder)
+clamp_to_window (GdkRectangle *out, GdkWindow *window, GdkRectangle *in)
 {
-  verbose_print (_("Preparing recording. Will start in 1 second...\n"));
-  byzanz_recorder_prepare (recorder);
-  g_timeout_add (1000, start_recording, recorder);
-  
-  return FALSE;
+  GdkRectangle window_area = { 0, };
+
+  gdk_drawable_get_size (GDK_DRAWABLE (window), &window_area.width, &window_area.height);
+  return gdk_rectangle_intersect (in, &window_area, in);
 }
 
 int
 main (int argc, char **argv)
 {
-  ByzanzRecorder *rec;
+  ByzanzSession *rec;
   GOptionContext* context;
-  GdkWindow *window;
   GError *error = NULL;
+  GFile *file;
   
   g_set_prgname (argv[0]);
 #ifdef GETTEXT_PACKAGE
-  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 #endif
 
   g_thread_init (NULL);
   context = g_option_context_new (_("record your current desktop session"));
+#ifdef GETTEXT_PACKAGE
+  g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
+#endif
+
   g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
   if (!g_option_context_parse (context, &argc, &argv, &error)) {
@@ -125,30 +145,23 @@ main (int argc, char **argv)
     usage ();
     return 0;
   }
-  window = gdk_get_default_root_window ();
-  if (gdk_drawable_get_depth (window) != 24 && 
-      gdk_drawable_get_depth (window) != 32) {
-    g_print (_("Byzanz requires a 24bpp or 32bpp depth for recording. "
-	  "The current resolution is set to %dbpp."), gdk_drawable_get_depth (window));
-    return 2;
-  }
-  rec = byzanz_recorder_new (argv[1], gdk_get_default_root_window (),
-      &area, loop, cursor);
-  if (rec == NULL) {
-    g_print (_("Could not prepare recording.\n"
-	  "Most likely the Damage extension is not available on the X server "
-	  "or the file \"%s\" is not writable.\n"), 
-	argv[1]);
-    return 2;
+  if (!clamp_to_window (&area, gdk_get_default_root_window (), &area)) {
+    g_print (_("Given area is not inside desktop.\n"));
+    return 1;
   }
+  file = g_file_new_for_commandline_arg (argv[1]);
+  rec = byzanz_session_new (file, byzanz_encoder_get_type_from_file (file),
+      gdk_get_default_root_window (), &area, cursor, audio);
+  g_object_unref (file);
+  g_signal_connect (rec, "notify", G_CALLBACK (session_notify_cb), NULL);
   delay = MAX (delay, 1);
   delay = (delay - 1) * 1000;
   duration = MAX (duration, 0);
   duration *= 1000;
-  g_timeout_add (delay, prepare_recording, rec);
+  g_timeout_add (delay, start_recording, rec);
   
   gtk_main ();
 
-  byzanz_recorder_destroy (rec);
+  g_object_unref (rec);
   return 0;
 }
